﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class GameController : MonoBehaviour {

	int currentLevel;

	bool inPause = false;
	bool isPausable = true;

	Canvas pauseMenuCanvas;
	Canvas levelOneCanvas;
	Canvas levelTwoCanvas;
	Canvas levelThreeCanvas;
	Text objectiveText;

	void Awake(){
		initComponents ();
		currentLevel = 1;
		hidePauseMenu ();
		hideLevelOneMenu ();
		hideLevelTwoMenu ();
		hideLevelThreeMenu ();
	}

	void Update(){
		if (isPausable) {
			detectPause ();
		}
	}

	void initComponents (){
		pauseMenuCanvas = getPauseCanvas ();
		levelOneCanvas = getLevelOneCanvas ();
		levelTwoCanvas = getLevelTwoCanvas ();
		levelThreeCanvas = getLevelThreeCanvas ();
		objectiveText = getObjectiveText ();
	}

	void detectPause(){
		if (Input.GetKeyDown (KeyCode.Escape)) {
			if (inPause) {
				resumeGame ();
				hidePauseMenu ();
			} else {
				pauseGame ();
				showPauseMenu ();
			}
		}
	}

	public void pauseGame(){
		inPause = true;
		Time.timeScale = 0;
	}

	public void resumeGame(){
		inPause = false;
		Time.timeScale = 1f;
	}

	public void setGameNoPausable(){
		isPausable = false;
	}

	public void setGamePausable(){
		isPausable = true;
	}

	void showPauseMenu(){
		pauseMenuCanvas.enabled = true;
	}

	void hidePauseMenu(){
		pauseMenuCanvas.enabled = false;
	}

	Canvas getPauseCanvas(){
		GameObject pauseMenu = GameObject.FindGameObjectWithTag ("PauseMenu");
		return pauseMenu.GetComponent<Canvas> ();
	}

	Canvas getLevelOneCanvas(){
		GameObject levelOne = GameObject.FindGameObjectWithTag ("LevelOneMenu");
		return levelOne.GetComponent<Canvas> ();
	}

	Canvas getLevelTwoCanvas(){
		GameObject levelTwo = GameObject.FindGameObjectWithTag ("LevelTwoMenu");
		return levelTwo.GetComponent<Canvas> ();
	}

	Canvas getLevelThreeCanvas(){
		GameObject levelThree = GameObject.FindGameObjectWithTag ("LevelThreeMenu");
		return levelThree.GetComponent<Canvas> ();
	}

	Text getObjectiveText(){
		GameObject objectiveTextObject = GameObject.FindGameObjectWithTag ("ObjectiveText");
		return objectiveTextObject.GetComponent<Text> ();
	}

	public bool isGamePaused(){
		return inPause;
	}

	public void manageLevelOne(){
		setGameNoPausable ();
		pauseGame ();
		showLevelOneMenu ();
	}

	public void manageLevelTwo(){
		setGameNoPausable ();
		pauseGame ();
		showLevelTwoMenu ();
	}

	public void manageLevelThree(){
		setGameNoPausable ();
		pauseGame ();
		showLevelThreeMenu ();
	}

	public void returnFromLevel(){
		setGamePausable ();
		resumeGame ();
		if (currentLevel == 1) {
			hideLevelOneMenu ();
			destroyLevelOneObjects ();
			currentLevel = 2;
			objectiveText.text = "Objetivo: Encuentra el diario de Tom Riddle";
		} else if (currentLevel == 2) {
			hideLevelTwoMenu ();
			destroyLevelTwoObjects ();
			currentLevel = 3;
			objectiveText.text = "Objetivo: Hazte con la copa de los tres magos";
		} else if (currentLevel == 3) {
			hideLevelThreeMenu ();
			destroyLevelThreeObjects ();
			objectiveText.text = "ENHORABUENA! Has terminado el juego";
			showPauseMenu ();
			pauseGame ();
		}
	}

	void showLevelOneMenu(){
		levelOneCanvas.enabled = true;
	}

	void hideLevelOneMenu(){
		levelOneCanvas.enabled = false;
	}

	void showLevelTwoMenu(){
		levelTwoCanvas.enabled = true;
	}

	void hideLevelTwoMenu(){
		levelTwoCanvas.enabled = false;
	}

	void showLevelThreeMenu(){
		levelThreeCanvas.enabled = true;
	}

	void hideLevelThreeMenu(){
		levelThreeCanvas.enabled = false;
	}

	void destroyLevelOneObjects(){
		GameObject piedraFilosofalObject = GameObject.FindGameObjectWithTag ("PiedraFilosofal");
		DestroyImmediate (piedraFilosofalObject);
		GameObject levelOneCanvasObject = GameObject.FindGameObjectWithTag ("LevelOneMenu");
		DestroyImmediate (levelOneCanvasObject);
	}

	void destroyLevelTwoObjects(){
		GameObject diarioObject = GameObject.FindGameObjectWithTag ("DiarioTomRiddle");
		DestroyImmediate (diarioObject);
		GameObject levelTwoCanvasObject = GameObject.FindGameObjectWithTag ("LevelTwoMenu");
		DestroyImmediate (levelTwoCanvasObject);
	}

	void destroyLevelThreeObjects(){
		GameObject CopaObject = GameObject.FindGameObjectWithTag ("CopaTresMagos");
		DestroyImmediate (CopaObject);
		GameObject levelThreeCanvasObject = GameObject.FindGameObjectWithTag ("LevelThreeMenu");
		DestroyImmediate (levelThreeCanvasObject);
	}

	public bool canAccessToLevel(int otherLevel){
		return otherLevel <= currentLevel;
	}

	void setCurrentLevel(int nextLevel){
		currentLevel = nextLevel;
	}

	public void exitGame(){
		Application.Quit ();
	}
}
