﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Assertions;

public class Warp : MonoBehaviour {

	public GameObject target;
	public GameObject targetMap;
	public int workingLevel;

	bool start = false;
	bool isFadeIn = false;
	float alpha = 0;
	float fadeTime = 1f; // Transition lasts 1 second
	GameController gameController;

	void Awake() {
		initComponents ();
		assertAllSetUp ();
		hideSprites ();
	}

	IEnumerator OnTriggerEnter2D(Collider2D other){
		if (isPlayer(other) && gameController.canAccessToLevel(workingLevel)) {
			disablePlayer (other);
			fadeIn ();
			yield return new WaitForSeconds (fadeTime);
			movePlayerToTarget (other);
			setUpCameraToTargetMap ();
			fadeOut ();
			enablePlayer (other);
			gameController.resumeGame();
		}
	}

	void OnGUI(){
		if (!start) {
			return;
		}

		Color color = new Color (GUI.color.r, GUI.color.g, GUI.color.b, alpha);
		GUI.color = color;

		// Creates a texture 1x1px black
		Texture2D texture = new Texture2D (1, 1); // Parameter are width and height
		texture.SetPixel(0, 0, Color.black);
		texture.Apply ();

		// Draws texture in the whole screen
		GUI.DrawTexture (new Rect (0, 0, Screen.width, Screen.height), texture);

		if (isFadeIn) {
			alpha = Mathf.Lerp (alpha, 1.1f, fadeTime * Time.deltaTime); // Get Dark
		} else {
			alpha = Mathf.Lerp (alpha, -0.1f, fadeTime * Time.deltaTime); // Get back normal
			if (alpha < 0)
				start = false;
		}
	}

	void initComponents(){
		GameObject gameControllerObject = GameObject.FindGameObjectWithTag ("GameController");
		gameController = gameControllerObject.GetComponent<GameController> ();
	}

	void disablePlayer(Collider2D playerCollider){
		Animator playerAnimator = playerCollider.GetComponent<Animator> ();
		PlayerController playerController = playerCollider.GetComponent<PlayerController> ();
		playerAnimator.enabled = false;
		playerController.enabled = false;
	}

	void enablePlayer(Collider2D playerCollider){
		Animator playerAnimator = playerCollider.GetComponent<Animator> ();
		PlayerController playerController = playerCollider.GetComponent<PlayerController> ();
		playerAnimator.enabled = true;
		playerController.enabled = true;
	}

	void fadeIn(){
		isFadeIn = true;
		start = true;
	}

	void fadeOut(){
		isFadeIn = false;
	}

	void assertAllSetUp (){
		Assert.IsNotNull (target);
		Assert.IsNotNull (targetMap);
	}

	bool isPlayer(Collider2D element){
		return element.tag == "Player";
	}

	void movePlayerToTarget(Collider2D element){
		Transform exit = getExitFromTarget ();
		element.transform.position = exit.position; 
	}

	Transform getExitFromTarget(){
		return target.transform.GetChild (0);
	}

	void hideSprites(){
		hideEntrySprite ();
		hideExitSprite ();
	}

	void hideEntrySprite(){
		SpriteRenderer entrySpriteRenderer = GetComponent<SpriteRenderer> ();
		entrySpriteRenderer.enabled = false;
	}

	void hideExitSprite(){
		Transform transformOfExit = getOwnExit();
		SpriteRenderer exitSpriteRenderer = getSpriteRendererFromTransform (transformOfExit);
		exitSpriteRenderer.enabled = false;
	}

	SpriteRenderer getSpriteRendererFromTransform(Transform transform){
		return transform.GetComponent<SpriteRenderer> ();
	}

	Transform getOwnExit(){
		return transform.GetChild (0);
	}

	void setUpCameraToTargetMap (){
		MainCameraController cameraController = Camera.main.GetComponent<MainCameraController> ();
		cameraController.setCameraBounds (targetMap);
	}
}
